@@ -9,90 +9,127 @@ Referenc:
  RPS server info on TDS300
   Get RPC info
    program vers proto   port
-    395183    1   tcp   1008
-    395184    1   tcp   1005
+    395183    1   tcp   1008 # VXI11-core
+    395184    1   tcp   1005 # VXI11-async
+
+DPO4034B:
+Starting Nmap 6.40-2 ( http://nmap.org ) at 2013-12-17 09:40 JST
+Nmap scan report for 192.168.2.4
+Host is up (0.00014s latency).
+Not shown: 996 closed ports
+PORT     STATE SERVICE
+80/tcp   open  http
+81/tcp   open  hosts2-ns -> e-Scope 
+111/tcp  open  rpcbind
+1001/tcp open  unknown -> VXI11-asyn
+MAC Address: 08:00:11:1F:38:64 (Tektronix)
+
+Nmap done: 1 IP address (1 host up) scanned in 0.20 seconds
+[noboru-mbookpro:~] noboru% rpcinfo -p 192.168.2.4
+   program vers proto   port
+    100000    2   tcp    111  portmapper
+    100000    2   udp    111  portmapper
+    395183    1   tcp    998  # VXI11-core
+    395184    1   tcp   1001  # VXI11-asyn
+
 """
 from TekOSC import *
-from  TekOSC import waveform as _waveform
+from TekOSC import waveform as _waveform
 
 class TekDPO(TekOSC):
-    def __init__(self,host,device="inst0,0"):
-        TekOSC.__init__(self,host,device)
-        self.write("VERBOSE OFF;")
+    # for DPO4K device="inst0"
+    def __init__(self,host,device="inst0,0",proto="tcp"):
+        TekOSC.__init__(self,host,device,proto)
+        self.write(b"VERBOSE OFF;")
         
     def get_cursor(self):
-        self.write(":CURS?;")
-        resp=self.read(4096)[:-1]
+        self.write(b":CURS?;")
+        resp=self.read(requestSize=4096)[:-1]
         return DPOCursor(resp)
     
-    def get_waveform(self,ch=1):
+    def get_waveform(self,ch=1,io_timeout=3000,requestSize=4096):
         if (type(ch) == types.StringType):
-            self.write(":DAT:SOU %s;"%ch)
+            self.write(b":DAT:SOU %s;"%ch)
         else:
-            self.write(":DAT:SOU CH%d;"%ch)
-        h=self.ask(":WFMO?;") # this opeation may take 0.3 sec avg.
-        self.write(":curve?;")
+            self.write(b":DAT:SOU CH%d;"%ch)
+        h=self.ask(b":WFMO?;") # this opeation may take 0.3 sec avg.
+        self.write(b":curve?;")
         #time.sleep(0.3) # need to wait until DPO is ready to send
-        r=self.readResponce(io_timeout=30)
+        r=self.readResponce(io_timeout=io_timeout, requestSize=requestSize)
         if r:
             wf=waveform(h,r)
         else:
-            raise exceptions.IOError,"No value"
+            raise IOError("No value")
         return wf
 
-    def get_curve(self,ch=1):
-        self.write(":DAT:SOU CH%d;:CURV?;"%ch)
+    def get_curve(self,ch=1,io_timeout=3000, requestSize=4096):
+        self.write(b":DAT:SOU CH%d;:CURV?;"%ch)
         #time.sleep(0.5) # need to wait until DPO is ready to send
-        return self.readResponce(io_timeout=30)
+        return self.readResponce(io_timeout=io_timeout,
+                                requestSize=requestSize)
 
-    def set_fulldata(self):
-        self.write("HOR:RECO 5000000")
-        self.write(":DATA:START 1;")
-        self.write(":DATA:STOP 5000000;")
+    def set_fulldata(self): # well, it is not quite correct.
+        #
+        self.write(b"HOR:RECO 10000000;")
+        self.write(b":DATA:START 1;")
+        self.write(b":DATA:STOP 10000000;")
 
     def set_wf_binary(self,fmt="RIBIN"):
-        self.write(":DATA:ENC %s;"%fmt)
+        self.write(b":DATA:ENC %s;"%fmt)
         
     def set_wf_ascii(self):
-        self.write(":DATA:ENC ASCI;")
+        self.write(b":DATA:ENC ASCI;")
 
     def set_ENC(self, mode):
-        enc=("ASCIi","FAStest","RIBinary","RPBinary","SRIbinary","SRPbinary")
+        #enc=("ASCIi","FAStest",
+        #"RIBinary","RPBinary",
+        #"SRIbinary","SRPbinary")
         enc=("ASCI","RIB","RPB","SRI","SRP")
         if (mode in enc):
-            self.write(":DATA:ENC %s;"%mode)
+            self.write(b":DATA:ENC %s;"%mode)
         elif type(mode) is types.StringType:
-            self.write(":DATA:ENC %s;"%mode)
+            self.write(b":DATA:ENC %s;"%mode)
         else:
-            self.write(":DATA:ENC %s;"%enc[mode])
+            self.write(b":DATA:ENC %s;"%enc[mode])
 
     def set_byte_width(self,nb):
         "nb should be 1 or 2"
-        self.write(":WFMO:BYT_N %d"%nb)
+        if (nb == 1) or (nb ==2):
+            self.write(b":WFMO:BYT_N %d"%nb)
+        else:
+            raise ValueError("argument out of range")
         
+    def set_record_length(self,recl):
+        #recl:100,10000,100000,5000000 for DPO3k(?)
+        #recl:100,10000,100000,10000000,20000000 for DPO4k 
+        self.write(b"HOR:RECO %d"%recl)
+        self.write(b"WFMINPRE:NR_PT %d"%recl)
+        self.write(b":DATA:START 1")
+        self.write(b":DATA:STOP %d"%recl)
+
     def get_SESR(self):
-        self.write("*ESR?;")
+        self.write(b"*ESR?;")
         return SESR(int(self.read()))
 
     def get_SBR(self):
-        self.write("*STB?;")
+        self.write(b"*STB?;")
         return SBR(int(self.read()))
 
     def get_DESER(self):
-        self.write(":DESE?;")
+        self.write(b":DESE?;")
         return DESER(self.read())
 
     def get_ESER(self):
-        self.write("*ESE?;")
+        self.write(b"*ESE?;")
         return ESER(self.read())
 
     def get_SRER(self):
-        self.write("*SRE?;")
+        self.write(b"*SRE?;")
         return SRER(self.read())
 
-    def get_measurement(self):
-        self.write(":MEASU?;")
-        return self.readResponce()
+    def get_measurement(self,requestSize=4096):
+        self.write(b":MEASU?;")
+        return self.readResponce(requestSize=requestSize)
 
     def qACQ(self):
         labels=(
@@ -103,7 +140,7 @@ class TekDPO(TekOSC):
             'NUMAVG',
             'SAMPLINGMODE'
             )
-        acq=self.ask(":ACQ?")
+        acq=self.ask(b":ACQ?")
         d=self.respToDict(labels,acq)
         if d:
             self.__dict__.update(d)
@@ -124,6 +161,7 @@ def bit(n,d):
 class Register:
     def __init__(self,ini_data):
         self.val=int(ini_data)
+
     def __str__(self):
         s=""
         for item in self.__dict__.items():
@@ -212,14 +250,14 @@ class waveform(_waveform):
     Preample for TDS3k:
         
     preamble for DPO3k:
-    >>> dpo3k.ask(":WFMO?;")
+    >>> dpo3k.ask(b":WFMO?;")
     '1;8;BIN;RI;MSB;"Ch1, DC coupling, 5.000mV/div, 400.0us/div, 10000 points, Sample mode";10000;Y;"s";400.0000E-9;-2.0000E-3;0;"V";200.0000E-6;0.0E+0;0.0E+0\n'
 
     preamble for DPO7k:
->>> dpo7k.ask(":WFMO?;")
-'2;16;BIN;RI;MSB;"Ch1, DC coupling, 1.000mV/div, 1.000us/div, 10000 points, Sample mode";10000;Y;"s";1.0000E-9;0.0000;5000;"V";156.2500E-9;18.4320E+3;0.0000;1\n'
+    >>> dpo7k.ask(b":WFMO?;")
+    '2;16;BIN;RI;MSB;"Ch1, DC coupling, 1.000mV/div, 1.000us/div, 10000 points, Sample mode";10000;Y;"s";1.0000E-9;0.0000;5000;"V";156.2500E-9;18.4320E+3;0.0000;1\n'
     """
-    
+
     def __init__(self,preamble,wfdata):
         """
         Examples WAVFRM? might return the waveform data as:(VERBOSE 0, header 1)
@@ -232,8 +270,8 @@ class waveform(_waveform):
         self.rdata=preamble.split(";")
         self.rdata.append(wfdata)
         if (len(self.rdata) < 17):
-            print "data read:",self.rdata[:16],len(self.rdata)
-            raise exceptions.ValueError,"Not enough data"
+            sys.stderr.write(b"%s data read:"%(self.rdata[:16], len(self.rdata)))
+            raise ValueError("Not enough data")
 
         self.byte_width=int(self.rdata[0])
         self.bit_width=int(self.rdata[1])
@@ -249,9 +287,10 @@ class waveform(_waveform):
         self.Point_Offset=int(self.rdata[11])
         self.Y_Unit=self.rdata[12]
         self.Y_Mult=float(self.rdata[13])
-        self.Y_Zero=float(self.rdata[14])
-        self.Y_Offset=float(self.rdata[15])
-        # curve start from rdata[16]
+        self.Y_Offset=float(self.rdata[14])
+        self.Y_Zero=float(self.rdata[15])
+
+        # curve start from rdata[16] < wrong! On newer models, header may contains more than 17 parameters.
         if (self.ENC == "ASCII" or self.ENC == "ASC"):
             self.conv_fmt=float
             self.wfsize=""
@@ -269,7 +308,7 @@ class waveform(_waveform):
                 self.raw=wfdata[:-1]
         #
         self._convert()
-        
+
     def _convert(self):
         if (self.ENC == "ASCII" or self.ENC == "ASC"):
             self.wf=[float(x) for x in self.raw.split(",")]
@@ -342,30 +381,54 @@ class DPOCursor(TekCursor):
         self.xy_ratio_unit=ent[35]
         self.xy_ratio=(float(ent[33]),float(ent[34]),float(ent[36]))
 
-def test(hostip="10.8.46.20"):
-    import Gnuplot
-    tek=TekOSC(hostip)
-    tek.set_fulldata()
-    gp=Gnuplot.Gnuplot()
-    return test_run(tek,gp)
+# def test(hostip="10.8.46.20",device="inst0"):
+#     import Gnuplot
+#     tek=TekOSC(hostip,device=device)
+#     #tek.set_fulldata()
+#     gp=Gnuplot.Gnuplot()
+#     return (tek,gp)
 
-def test_run(tek,gp):
+def test_run(tek):
     runstate=tek.qAcqState()
     stopafter=tek.qAcqStopAfter()
     tek.Stop()
     tek.set_AcqStopAfter("SEQ")
     tek.Run();tek.qOPC()
-    wf1,wf2=test_update(tek,gp)
+    wf1,wf2=test_update(tek)
     tek.set_AcqStopAfter(stopafter)
     tek.set_AcqState(runstate)
-    return (tek,wf1,wf2,gp)
+    return (wf1,wf2)
 
-def test_update(tek,gp):
+def test_update(tek):
     wf1=tek.get_waveform(1)
     wf2=tek.get_waveform(2)
-    gp.title("Python VXI-11 module example from %s"%tek.Model)
-    gp.plot(wf1.trace(),wf2.trace())
     return (wf1,wf2)
+
+def test_srq(hostname="192.168.2.4", device="inst0"):
+    """
+    To generate a service request (SRQ) interrupt to an external controller, at least one bit in the Status Byte Register must be enabled. These bits are enabled by using the *SRE common command to set the corresponding bit in the Service Request Enable Register. These enabled bits can then set RQS and MSS (bit 6) in the Status Byte Register.
+    """
+    import select,threading,atexit
+    osc=tek=TekOSC(hostname,device)
+    #osc.create_intr_chan(proto=cVXI11.Device_AddrFamily.DEVICE_UDP)
+    osc.createSVCThread()
+    sys.stdout.write(b"Thread Created\n")
+    osc.Stop()
+    #osc.set_wf_point_mode("MAX")
+    #osc.set_wf_point("5000")
+    osc.write(b":DESE 1") #OPC
+    osc.write(b"*ESE 1") #Pon(7)/URQ(6)/CME(5)/EXE(4)/DDE(3)/QYE(2)/RQL(1)/OPC(0)
+    osc.write(b"*SRE 32") # SRE /ESB(5)/MAV(4)
+    osc.write(b"*SRE 48") # enable all SRQ source
+    osc.write(b"*OPC")
+    osc.enable_srq()
+    #
+    osc.check_SRQ_source()
+    osc.svc_lock.acquire(Flase)
+    sys.stdout.write(b"%s"%osc.svc_lock.locked()) #should be True
+    osc.write(b":ACQ:STATE RUN;*OPC;")
+    sys.stdout.write(b"%s"%osc.svc_lock.locked())
+    return osc
+
 if __name__ == "__main__":
-    test()
-    
+    test("169.254.4.84")
