@@ -3,25 +3,34 @@
 """
 A module to support Tektronix TDS3000 series OSC. Mostly work with other OSC from Tektronix, with minor modification.
 (C) Noboru Yamamoto,2009. KEK, Ibaraki, JAPAN
+
 Referenc:
  "TDS3000,TDS3000B, and TDS3000C series Digital Phosphor Oscilloscopes", 071--381-03,Tektronix
 
- RPS server info on TDS300
+ RPC server info on TDS300
   Get RPC info
    program vers proto   port
     395183    1   tcp   1008
     395184    1   tcp   1005
 """
 
-import vxi11Device
-import time,types,struct,exceptions
+#import vxi11Device
+import cVXI11
+import time,types,struct,sys
+#exceptions
 
-class TekOSC(vxi11Device.Vxi11Device):
-    def __init__(self,host,device="inst0,0"):
-        vxi11Device.Vxi11Device.__init__(self, host,device)
+try:
+    import numpy
+    _use_numpy_fromstring=True
+except:
+    _use_numpy_fromstring=False
+    
+class TekOSC(cVXI11.IEEE488_2Device):
+    def __init__(self,host,device="inst0,0",proto="tcp"):
+        cVXI11.Vxi11Device.__init__(self, host,device,proto)
         self.IDN_Str=self.qIDN()
-        self.write("VERBOSE ON;")
-        self.write("HEADER OFF;")
+        self.write(b"VERBOSE ON;")
+        self.write(b"HEADER OFF;")
         ID=self.IDN_Str[:-1].split(",")
         self.Make=ID[0]
         self.Model=ID[1]
@@ -34,176 +43,208 @@ class TekOSC(vxi11Device.Vxi11Device):
         return d
         
     def qIDN(self):
-        self.write("*IDN?;")
+        self.write(b"*IDN?;")
         return self.read()
 
     def qACQ(self):
-        self.write("ACQ?;")
+        self.write(b"ACQ?;")
         return self.read()
 
     def qACQMODE(self):
-        self.write("ACQ:MODE?;")
+        self.write(b"ACQ:MODE?;")
         return self.read()
 
     def qNUMAVG(self):
-        self.write("ACQ:NUMAV?;")
+        self.write(b"ACQ:NUMAV?;")
         return int(self.read())
 
     def qNUMENV(self):
-        self.write("ACQ:NUMENV?;")
+        self.write(b"ACQ:NUMENV?;")
         return int(self.read())
 
     def qAcqState(self):
-        self.write("ACQ:STATE?;")
+        self.write(b"ACQ:STATE?;")
         return self.read()
 
     def qStopAfter(self):
-        self.write("ACQ:STOPA?;")
+        self.write(b"ACQ:STOPA?;")
         return self.read()
 
     def qAcqStopAfter(self):
-        self.write("ACQ:STOPA?;")
+        self.write(b"ACQ:STOPA?;")
         return self.read()
 
     def qAllEv(self):
-        self.write("ALLE?;")
+        self.write(b"ALLE?;")
         return self.read()
         
     def qBusy(self):
-        self.write("BUSY?;")
+        self.write(b"BUSY?;")
         return int(self.read())
 
     def qCH(self,ch=1):
-        self.write("CH%d?;"%ch)
+        self.write(b"CH%d?;"%ch)
         return self.read()
         
     def qDESE(self):
-        self.write("DESE?")
+        self.write(b"DESE?")
         return self.read()
 
     def qENC(self):
-        self.write("DATA:ENC?")
+        self.write(b"DATA:ENC?")
         return self.read()
 
     def qVERBOSE(self):
-        return self.ask(":VERBOSE?")
+        return self.ask(b":VERBOSE?")
 
     def qHEADER(self):
-        return self.ask(":HEADER?")
+        return self.ask(b":HEADER?")
 
     def CLS(self):
-        self.write("*CLS;\n")
+        self.write(b"*CLS;\n")
 
     def OPC(self):
-        self.write("*OPC;")
+        self.write(b"*OPC;")
 
     def qOPC(self,io_timeout=10000):
-        self.write("*OPC?;")
+        self.write(b"*OPC?;")
         return self.read(io_timeout=io_timeout)
 
     def Run(self):
-        self.write(":ACQ:STATE RUN;")
+        self.write(b":ACQ:STATE RUN;")
 
     def Stop(self):
-        self.write(":ACQ:STATE STOP;")
+        self.write(b":ACQ:STATE STOP;")
 
     def set_AcqState(self,state):
         if state in ("ON", "OFF","RUN","STOP","0","1"):
-            self.write("ACQ:STATE %s;"%state)
+            self.write(b"ACQ:STATE %s;"%state)
         elif state[:-1] in ("ON", "OFF","RUN","STOP","0","1"):
-            self.write("ACQ:STATE %s;"%state[:-1])
+            self.write(b"ACQ:STATE %s;"%state[:-1])
 
     def set_AcqStopAfter(self,state):
-        if state in ("RUNST", "SEQ","RUNSTOP","SEQUENCE"):
-            self.write(":ACQ:STOPA %s;"%state)
+        if state in ("RUNST", "SEQ", "RUNSTOP","SEQUENCE"):
+            self.write(b":ACQ:STOPA %s;"%state)
         elif state[:-1] in ("RUNST", "SEQ","RUNSTOP","SEQUENCE"):
-            self.write(":ACQ:STOPA %s;"%state[:-1])
+            self.write(b":ACQ:STOPA %s;"%state[:-1])
         
     def clear(self):
-        self.write("*CLS;")
+        self.write(b"*CLS;")
         
     def wait(self):
-        self.write("*WAI;")
+        self.write(b"*WAI;")
         
     def device_clear(self,flags=0,lock_timeout=0,io_timeout=5):
-        vxi11Device.Vxi11Device.clear(self, flags, lock_timeout, io_timeout)
+        cVXI11.Vxi11Device.clear(self, flags, lock_timeout, io_timeout)
 
     def clear_all(self):
         self.clear()
         self.device_clear()
         
     def get_cursor(self):
-        self.write(":CURS?;")
+        self.write(b":CURS?;")
         resp=self.read(4096)[:-1]
         return TekCursor(resp)
     
-    def get_waveform(self,ch=1):
+    def get_waveform(self,ch=1,requestSize=4096,io_timeout=3000):
         """
-        it is better to stop scanning before get waveform data on TDS.
+        it seems better to stop scanning before getting waveform data from TDS.
         """
-        if (type(ch) ==types.StringType):
-            self.write(":DAT:SOU %s;:WAVFRM?"%ch)
-        else:
-            self.write(":DAT:SOU CH%d;:WAVFRM?;"%ch)
-        r=self.readResponce(io_timeout=1)
+        self.lock()
+        try:
+            if (type(ch) ==types.StringType):
+                self.write(b":DAT:SOU %s;"%ch)
+            else:
+                self.write(b":DAT:SOU CH%d;"%ch)
+            ts=time.time()
+            self.write(b":WAVFRM?;");time.sleep(0.2) #0.2 sec seems a magic number for TDS3K
+            r=self.readResponce(io_timeout=io_timeout,
+                                requestSize=requestSize)
+        finally:
+            self.unlock()
         wf=waveform(r)
+        wf.TS=ts
         return wf
 
-    def get_curve(self,ch=1):
-        self.write(":DAT:SOU CH%d;:CURV?;"%ch)
-        return self.readResponce(io_timeout=5)
+    def get_curve(self,ch=1,requestSize=4096,io_timeout=3000):
+        self.write(b":DAT:SOU CH%d;:CURV?;"%ch)
+        return self.readResponce(io_timeout=io_timeout,
+                                requestSize=requestSize)
 
     def set_fulldata(self):
-        self.write("HOR:RECORDL 10000")
-        self.write(":DATA:START 1;")
-        self.write(":DATA:STTOP 10000;")
+        self.write(b"HOR:RECORDL 10000")
+        self.write(b":DATA:START 1;")
+        self.write(b":DATA:STTOP 10000;")
 
     def set_wf_binary(self,fmt="RIBIN"):
-        self.write(":DATA:ENC %s;"%fmt)
+        self.write(b":DATA:ENC %s;"%fmt)
         
     def set_wf_ascii(self):
-        self.write(":DATA:ENC ASCI;")
+        self.write(b":DATA:ENC ASCI;")
 
     def set_ENC(self, mode):
         enc=("ASCIi","FAStest","RIBinary","RPBinary","SRIbinary","SRPbinary")
         enc=("ASCI","RIB","RPB","SRI","SRP")
         if (mode in enc):
-            self.write(":DATA:ENC %s;"%mode)
+            self.write(b":DATA:ENC %s;"%mode)
         elif type(mode) is types.StringType:
-            self.write(":DATA:ENC %s;"%mode)
+            self.write(b":DATA:ENC %s;"%mode)
         else:
-            self.write(":DATA:ENC %s;"%enc[mode])
+            self.write(b":DATA:ENC %s;"%enc[mode])
+
+    def select_data_source(self,source):
+        if type(source) == type(1):
+            self.write(b":DATA:SOU CH%d;"%source)
+        else:
+            self.write(b":DATA:SOU %s;"%source)
 
     def get_SESR(self):
-        self.write("*ESR?;")
+        self.write(b"*ESR?;")
         return SESR(int(self.read()))
 
     def get_SBR(self):
-        self.write("*STB?;")
+        self.write(b"*SBR?;")
         return SBR(int(self.read()))
 
     def get_DESER(self):
-        self.write(":DESE?;")
+        self.write(b":DESE?;")
         return DESER(self.read())
 
     def get_ESER(self):
-        self.write("*ESE?;")
+        self.write(b"*ESE?;")
         return ESER(self.read())
 
     def get_SRER(self):
-        self.write("*SRE?;")
+        self.write(b"*SRE?;")
         return SRER(self.read())
 
     def get_measurement(self):
-        self.write(":MEASU?;")
+        self.write(b":MEASU?;")
         return self.readResponce()
 
+    def reset_PSC(self): # reset Enable Registers (DESER/ESER/SRER)
+        self.write(b"*PSC 1")
+
+    def save_PSC(self): # reset Enable Registers (DESER/ESER/SRER)
+        self.write(b"*PSC 0")
+
+
+    def check_SRQ_source(self):
+        # after SRQ occured, srq source registers should be examined to reset
+        sbr=int(self.ask(b"*STB?"))
+        # -/RQS(MSS)/ESB/MAV/-/-/-/-
+        sesr=int(self.ask(b"*ESR?")) # Standard Event Status Register
+        #SESR:PON/URQ/CME/EXE/DDE/QYE/RQC/OPC
+        evnt=self.ask(b"ALLEV?")
+        return dict(SBR=sbr, SESR=sesr,EVNT=evnt)
+
 def bit(n,d):
     return (d>>n)&1
     
 class Register:
     def __init__(self,ini_data):
         self.val=int(ini_data)
+        
     def __str__(self):
         s=""
         for item in self.__dict__.items():
@@ -289,12 +330,16 @@ class waveform:
     BFMT={'RI':"h",'RP':"H"}
     BORD={"MSB":">","LSB":"<"}
     def __init__(self,data):
+        """
+        make sure "VERBOSE" is off before getting preamble of waveform data.
+        """
         self.rdata=data.split(";")
-        if (len(self.rdata) < 17):
-            print self.rdata[:16],len(self.rdata)
-            raise exceptions.ValueError,"Not enough data"
+        if (len(self.rdata) < 17):# avoide the magick number  for future models.
+            sys.stderr.write(self.rdata[:16],len(self.rdata))
+            raise ValueError("Not enough data")
+        # shuld search for ":CURVE" for ascii "#" for binary data
         wfdata=";".join(self.rdata[16:])
-
+        #Bug. Order of data may differenc in the recent models.  
         self.byte_width=int(self.rdata[0])
         self.bit_width=int(self.rdata[1])
         self.ENC=self.rdata[2]
@@ -311,10 +356,11 @@ class waveform:
         self.Y_Zero=float(self.rdata[13])
         self.Y_Offset=float(self.rdata[14])
         self.Y_Unit=self.rdata[15]
+        self.TS=None
         if (self.ENC == "ASCII" or self.ENC == "ASC"):
             self.conv_fmt=float
             self.wfsize=""
-            if (curve[0] == ":"):
+            if (wfdata[0] == ":"):
                 self.raw=wfdata[len(":CURVE "):-1]
             else:
                 self.raw=wfdata[:-1]
@@ -327,7 +373,6 @@ class waveform:
             else:
                 self.raw=wfdata[:-1]
         self._convert()
-
         
     def update(self,curve):
         if (self.ENC == "ASCII" or self.ENC == "ASC"):
@@ -348,10 +393,15 @@ class waveform:
     def _convert(self):
         if (self.ENC == "ASCII" or self.ENC == "ASC"):
             self.wf=[float(x) for x in self.raw.split(",")]
-        else:
-            self.wf=[struct.unpack(self.conv_fmt,self.raw[i:i+self.byte_width])[0]
+        else:# binary
+            # consider to use scipy.fromstring if scipy is avaialble
+            #if _use_numpy_fromstring:
+            #self.wf=numpy.fromstring(self.raw[i:i+self.byte_width],dtype=np.uint,count=len(self.raw)/self.byte_width,sep='')
+            #else
+            self.wf=[struct.unpack(self.conv_fmt,
+                                   self.raw[i:i+self.byte_width])[0]
                      for i in range(0,len(self.raw),self.byte_width)]
-        
+
         self.y=[((y-self.Y_Offset)*self.Y_Mult+self.Y_Zero) for y in self.wf]
         self.x=[ (x*self.X_Incr+self.X_Zero) for x in range(len(self.y))]
         
@@ -370,18 +420,17 @@ class TekCursor:
         self.hpos=(float(ent[8]),float(ent[9]))
         self.hbarspos=(float(ent[10]),float(ent[11]),float(ent[12]))
 
-# some utility function
-
-def TDS3000Reboot(addr):
-    s=socket.socekt()
-    s.sendto("GET /resetinst.cgi HTTP/1.0\n\n",(addr,80))
-    return
-
-# test function
 def test(hostip="10.9.16.20"):
     import Gnuplot
     tek=TekOSC(hostip)
     tek.set_fulldata()
+    # for SRQ
+    sys.stdout.write(b"%s"%tek.check_SRQ_source())
+    tek.write(b":DESE 1")
+    tek.write(b"*ESE 1")
+    tek.write(b"*SRE 32")
+    tek.createSVCThread()
+    tek.enable_srq()
     gp=Gnuplot.Gnuplot()
     return test_run(tek,gp)
 
@@ -402,7 +451,6 @@ def test_update(tek,gp):
     gp.title("Python VXI-11 module example from %s"%tek.Model)
     gp.plot(wf1.trace(),wf2.trace())
     return (wf1,wf2)
+
 if __name__ == "__main__":
     test()
-    
-
